@@ -203,7 +203,7 @@ struct _zvtprivate
 };
 
 /* *** DO NOT USE THIS IN APPS! *** */
-#define _ZVT_PRIVATE(term) ((struct _zvtprivate *)gtk_object_get_data (GTK_OBJECT (term), "_zvtprivate"))
+#define _ZVT_PRIVATE(term) ((struct _zvtprivate *)g_object_get_data (G_OBJECT (term), "_zvtprivate"))
 
 /* options for fork */
 #define ZVT_TERM_DO_UTMP_LOG 1
