@@ -28,6 +28,14 @@
 #endif
 #include <sys/types.h>
 
+/* we need GdkFont calls for now */
+#ifdef GDK_DISABLE_DEPRECATED
+#undef GDK_DISABLE_DEPRECATED
+#include <gdk/gdkfont.h>
+#include <gdk/gdkx.h>
+#define GDK_DISABLE_DEPRECATED
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
@@ -52,6 +60,11 @@
 #include <X11/Xos.h>
 
 #include "zvt-marshal.h"
+#include "zvt-accessible.h"
+#include "zvt-accessible-factory.h"
+
+#define READ_CONDITION (G_IO_IN | G_IO_HUP | G_IO_ERR)
+#define WRITE_CONDITION (G_IO_OUT | G_IO_ERR)
 
 /* define to 'x' to enable copious debug output */
 #define d(x)
@@ -91,24 +104,26 @@ static void zvt_term_title_changed_raise (void *user_data, char *str, VTTITLE_TY
 static void zvt_term_got_output (ZvtTerm *term, const gchar *buffer, gint count);
 static gint zvt_term_cursor_blink (gpointer data);
 static void zvt_term_scrollbar_moved (GtkAdjustment *adj, GtkWidget *widget);
-static void zvt_term_readdata (gpointer data, gint fd, GdkInputCondition condition);
-static void zvt_term_readmsg (gpointer data, gint fd, GdkInputCondition condition);
+static gboolean zvt_term_readdata (GIOChannel *source, GIOCondition condition, gpointer data);
+static gboolean zvt_term_readmsg (GIOChannel *source, GIOCondition condition, gpointer data);
 static void zvt_term_fix_scrollbar (ZvtTerm *term);
 static void vtx_unrender_selection (struct _vtx *vx);
 static void zvt_term_scroll (ZvtTerm *term, int n);
 static void zvt_term_scroll_by_lines (ZvtTerm *term, int n);
 static int vt_cursor_state(void *user_data, int state);
-static void zvt_term_writemore (gpointer data, gint fd, GdkInputCondition condition);
+static gboolean zvt_term_writemore (GIOChannel *source, GIOCondition condition, gpointer data);
 static void zvt_term_updated(ZvtTerm *term, int mode);
 static void clone_col(unsigned short **dest, unsigned short *from);
 static void zvt_term_real_copy_clipboard (ZvtTerm *term);
 static void zvt_term_real_paste_clipboard (ZvtTerm *term);
 static void zvt_term_real_selection_changed (ZvtTerm *term);
+static AtkObject *zvt_term_get_accessible (GtkWidget *widget);
 
 /* callbacks from update layer */
 void vt_draw_text(void *user_data, struct vt_line *line, int row, int col, int len, int attr);
 void vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill);
 static void vt_selection_changed(void *user_data);
+gint zvt_input_add (gint source, GIOCondition condition, GIOFunc function, gpointer data);
 
 /* static data */
 
@@ -149,6 +164,21 @@ static guint term_signals[LAST_SIGNAL] = { 0 };
 /* GTK parent class */
 static GtkWidgetClass *parent_class = NULL;
 
+gint
+zvt_input_add (gint             source,
+	       GIOCondition 	condition,
+	       GIOFunc  	function,
+	       gpointer         data)
+{
+  guint result;
+  GIOChannel *channel;
+      
+  channel = g_io_channel_unix_new (source);
+  result = g_io_add_watch (channel, condition, function, data);
+  g_io_channel_unref (channel);
+
+  return result;
+}
 
 guint
 zvt_term_get_type (void)
@@ -156,18 +186,23 @@ zvt_term_get_type (void)
   static guint term_type = 0;
   
   if (!term_type) {
-      GtkTypeInfo term_info = {
-	"ZvtTerm",
-	sizeof (ZvtTerm),
+      GTypeInfo term_info = {
 	sizeof (ZvtTermClass),
-	(GtkClassInitFunc) zvt_term_class_init,
-	(GtkObjectInitFunc) zvt_term_init,
-	NULL,
+	(GBaseInitFunc)0,
+	(GBaseFinalizeFunc)0,
+	(GClassInitFunc) zvt_term_class_init,
+	(GClassFinalizeFunc)0,
 	NULL,
+	sizeof (ZvtTerm),
+	0,
+	(GInstanceInitFunc) zvt_term_init,
 	NULL
       };
       
-      term_type = gtk_type_unique (gtk_widget_get_type (), &term_info);
+      term_type = g_type_register_static (gtk_widget_get_type (), 
+					  "ZvtTerm",
+					  &term_info,
+					  0);
     }
 
   return term_type;
@@ -177,85 +212,93 @@ zvt_term_get_type (void)
 static void
 zvt_term_class_init (ZvtTermClass *class)
 {
-  GtkObjectClass *object_class;
+  GObjectClass *object_class;
   GtkWidgetClass *widget_class;
   ZvtTermClass *term_class;
   GtkBindingSet  *binding_set;
 
-  object_class = (GtkObjectClass*) class;
+  object_class = (GObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   term_class = (ZvtTermClass*) class;
 
   parent_class = gtk_type_class (gtk_widget_get_type ());
 
   term_signals[CHILD_DIED] =
-    gtk_signal_new ("child_died",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (ZvtTermClass, child_died),
-                    gtk_signal_default_marshaller,
-                    GTK_TYPE_NONE, 0);
+    g_signal_new ("child_died",
+		  G_TYPE_FROM_CLASS (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (ZvtTermClass, child_died),
+		  NULL,
+		  NULL,
+		  zvt_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 
   term_signals[TITLE_CHANGED] =
-    gtk_signal_new ("title_changed",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (ZvtTermClass, title_changed),
-		    /*
-                    gnome_marshal_VOID__INT_POINTER,
-		    */
-                    zvt_marshal_VOID__INT_POINTER,
-                    GTK_TYPE_NONE, 2,
-                    GTK_TYPE_INT,
-		    GTK_TYPE_POINTER);
+    g_signal_new ("title_changed",
+		  G_TYPE_FROM_CLASS (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (ZvtTermClass, title_changed),
+		  NULL,
+		  NULL,
+		  zvt_marshal_VOID__INT_POINTER,
+		  G_TYPE_NONE, 2,
+		  G_TYPE_INT,
+		  G_TYPE_POINTER);
   
   term_signals[GOT_OUTPUT] =
-      gtk_signal_new ("got_output",
-		      GTK_RUN_FIRST,
-		      GTK_CLASS_TYPE (object_class),
-		      GTK_SIGNAL_OFFSET (ZvtTermClass, got_output),
-		      /*
-		      gnome_marshal_VOID__POINTER_INT,
-		      */
-		      gtk_marshal_VOID__POINTER_INT,
-		      GTK_TYPE_NONE, 2,
-		      GTK_TYPE_POINTER,
-		      GTK_TYPE_INT);
+      g_signal_new ("got_output",
+		    G_TYPE_FROM_CLASS (object_class),	
+		    G_SIGNAL_RUN_FIRST,
+		    G_STRUCT_OFFSET (ZvtTermClass, got_output),
+		    NULL,
+		    NULL,
+		    zvt_marshal_VOID__POINTER_INT,
+		    G_TYPE_NONE, 2,
+		    G_TYPE_POINTER,
+		    G_TYPE_INT);
       
   term_signals[COPY_CLIPBOARD] = 
-      gtk_signal_new ("copy_clipboard",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (ZvtTermClass, copy_clipboard),
-		    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+      g_signal_new ("copy_clipboard",
+		    G_TYPE_FROM_CLASS (object_class),	
+		    G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		    G_STRUCT_OFFSET (ZvtTermClass, copy_clipboard),
+		    NULL,
+		    NULL,
+		    zvt_marshal_VOID__VOID,
+		    G_TYPE_NONE, 0);
 
     term_signals[PASTE_CLIPBOARD] =
-    gtk_signal_new ("paste_clipboard",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (ZvtTermClass, paste_clipboard),
-		    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("paste_clipboard",
+		    G_TYPE_FROM_CLASS (object_class),	
+		    G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		    G_STRUCT_OFFSET (ZvtTermClass, paste_clipboard),
+		    NULL,
+		    NULL,
+		    zvt_marshal_VOID__VOID,
+		    G_TYPE_NONE, 0);
 
     term_signals[SCROLL] =
-    gtk_signal_new ("scroll",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (ZvtTermClass,scroll),
-		    gtk_marshal_VOID__INT,
-		    GTK_TYPE_NONE, 1, 
-		    GTK_TYPE_INT);
+    g_signal_new ("scroll",
+		  G_TYPE_FROM_CLASS (object_class), 
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (ZvtTermClass,scroll),
+		  NULL,
+		  NULL,
+		  zvt_marshal_VOID__INT,
+		  G_TYPE_NONE, 1, 
+		  G_TYPE_INT);
 
     term_signals[SELECTION_CHANGED] =
-    gtk_signal_new ("selection_changed",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (ZvtTermClass, selection_changed),
-		    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("selection_changed",
+		  G_TYPE_FROM_CLASS (object_class),
+		  G_SIGNAL_RUN_LAST,	
+		  G_STRUCT_OFFSET (ZvtTermClass, selection_changed),
+		  NULL,
+		  NULL,
+		  zvt_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 
-  object_class->destroy = zvt_term_destroy;
+  GTK_OBJECT_CLASS (object_class)->destroy = zvt_term_destroy;
 
   widget_class->realize = zvt_term_realize;
   widget_class->unrealize = zvt_term_unrealize;
@@ -270,6 +313,8 @@ zvt_term_class_init (ZvtTermClass *class)
   widget_class->button_press_event = zvt_term_button_press;
   widget_class->button_release_event = zvt_term_button_release;
   widget_class->motion_notify_event = zvt_term_motion_notify;
+  widget_class->get_accessible = zvt_term_get_accessible;
+  
   widget_class->scroll_event = zvt_term_scroll_event;
 
   widget_class->selection_clear_event = zvt_term_selection_clear;
@@ -386,7 +431,7 @@ term->vx->cursor_state = vt_cursor_state;
   zp->queue_green = 0;
   zp->queue_blue = 0;
 
-  gtk_object_set_data(GTK_OBJECT (term), "_zvtprivate", zp);
+  g_object_set_data(G_OBJECT (term), "_zvtprivate", zp);
 
   /* charwidth, charheight set here */
   zvt_term_set_font_name (term, DEFAULT_FONT);
@@ -394,13 +439,13 @@ term->vx->cursor_state = vt_cursor_state;
   /* scrollback position adjustment */
   term->adjustment =
     GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 1.0, 1.0, 1.0));
-  gtk_object_ref (GTK_OBJECT (term->adjustment));
+  g_object_ref (G_OBJECT (term->adjustment));
   gtk_object_sink (GTK_OBJECT (term->adjustment));
 
-  gtk_signal_connect (
-      GTK_OBJECT (term->adjustment),
+  g_signal_connect (
+      G_OBJECT (term->adjustment),
       "value_changed",
-      GTK_SIGNAL_FUNC (zvt_term_scrollbar_moved),
+      G_CALLBACK (zvt_term_scrollbar_moved),
       term);
 
   /* selection received */
@@ -579,7 +624,7 @@ GtkWidget*
 zvt_term_new_with_size (int cols, int rows)
 {
   ZvtTerm *term;
-  term = gtk_type_new (zvt_term_get_type ());
+  term = g_object_new (ZVT_TYPE_TERM, NULL);
 
   /* fudge the pixel size, not (really) used anyway */
   vt_resize (&term->vx->vt, cols, rows, cols*8, rows*8);
@@ -605,7 +650,7 @@ GtkWidget*
 zvt_term_new (void)
 {
   ZvtTerm *term;
-  term = gtk_type_new (zvt_term_get_type ());
+  term = g_object_new (ZVT_TYPE_TERM, NULL);
   return GTK_WIDGET (term);
 }
 
@@ -615,15 +660,14 @@ zvt_term_destroy (GtkObject *object)
   ZvtTerm *term;
   struct _zvtprivate *zp;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (ZVT_IS_TERM (object));
 
   term = ZVT_TERM (object);
   zp = _ZVT_PRIVATE(term);
 
   if (!zp)
-    return;
-
+    goto chain_destroy;
+  
   if (term->timeout_id != -1) {
     gtk_timeout_remove(term->timeout_id);
     term->timeout_id = -1;
@@ -645,8 +689,9 @@ zvt_term_destroy (GtkObject *object)
 
   /* release the adjustment */
   if (term->adjustment) {
-    gtk_signal_disconnect_by_data (GTK_OBJECT (term->adjustment), term);
-    gtk_object_unref (GTK_OBJECT (term->adjustment));
+    g_signal_handlers_disconnect_matched (term->adjustment, G_SIGNAL_MATCH_DATA,
+					  0, 0, NULL, NULL, term);
+    g_object_unref (term->adjustment);
     term->adjustment = NULL;
   }
 
@@ -663,11 +708,11 @@ zvt_term_destroy (GtkObject *object)
     if (zp->text_expand)
       g_free(zp->text_expand);
     if (zp->bold_save)
-      gdk_pixmap_unref(zp->bold_save);
+      g_object_unref(zp->bold_save);
     if (zp->paste)
       g_free(zp->paste);
     if (zp->paste_id != -1)
-      gdk_input_remove(zp->paste_id);
+      g_source_remove(zp->paste_id);
 
     if (zp->queue_red)
 	    g_free(zp->queue_red);
@@ -677,9 +722,10 @@ zvt_term_destroy (GtkObject *object)
 	    g_free(zp->queue_blue);
     
     g_free(zp);
-   gtk_object_set_data (GTK_OBJECT (term), "_zvtprivate", 0);
+   g_object_set_data (G_OBJECT (term), "_zvtprivate", 0);
   }
 
+ chain_destroy:
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -903,7 +949,7 @@ zvt_term_realize (GtkWidget *widget)
 			       &widget->style->bg[GTK_STATE_ACTIVE],
 			       0, 0);
   gdk_window_set_cursor(widget->window, term->cursor_bar);
-  gdk_pixmap_unref (cursor_dot_pm);
+  g_object_unref (cursor_dot_pm);
   zp->cursor_hand = gdk_cursor_new(GDK_HAND2);
   term->cursor_current = term->cursor_bar;
 
@@ -975,13 +1021,13 @@ zvt_term_unrealize (GtkWidget *widget)
   zp = _ZVT_PRIVATE(term);
 
   /* free resources */
-  gdk_cursor_destroy (term->cursor_bar);
+  gdk_cursor_unref (term->cursor_bar);
   term->cursor_bar = NULL;
 
-  gdk_cursor_destroy (term->cursor_dot);
+  gdk_cursor_unref (term->cursor_dot);
   term->cursor_dot = NULL;
 
-  gdk_cursor_destroy (zp->cursor_hand);
+  gdk_cursor_unref (zp->cursor_hand);
   zp->cursor_hand = NULL;
 
   term->cursor_current = NULL;
@@ -991,16 +1037,16 @@ zvt_term_unrealize (GtkWidget *widget)
   gdk_color_context_free (term->color_ctx);
   term->color_ctx = NULL;
   */
-  gdk_colormap_unref(term->color_map);
+  g_object_unref(term->color_map);
 
 
-  gdk_gc_destroy (term->scroll_gc);
+  g_object_unref (term->scroll_gc);
   term->scroll_gc = NULL;
 
-  gdk_gc_destroy (term->back_gc);
+  g_object_unref (term->back_gc);
   term->back_gc = NULL;
 
-  gdk_gc_destroy (term->fore_gc);
+  g_object_unref (term->fore_gc);
   term->fore_gc = NULL;
 
   if (GTK_WIDGET_CLASS (parent_class)->unrealize)
@@ -1243,10 +1289,11 @@ zvt_term_expose (GtkWidget      *widget,
     term->in_expose = 0;
 
     if (term->shadow_type != GTK_SHADOW_NONE)
-      gtk_draw_shadow (widget->style, widget->window,
-		       GTK_STATE_NORMAL, term->shadow_type, 0, 0, 
-		       widget->allocation.width,
-		       widget->allocation.height); 
+      gtk_paint_shadow (widget->style, widget->window,
+		        GTK_STATE_NORMAL, term->shadow_type, 
+			NULL, widget, NULL, 0, 0, 
+		        widget->allocation.width,
+		        widget->allocation.height); 
   }
 
   return FALSE;
@@ -1372,7 +1419,7 @@ zvt_term_set_fonts_internal(ZvtTerm *term, GdkFont *font, GdkFont *font_bold)
     int depth;
 
     if (zp->bold_save)
-      gdk_pixmap_unref(zp->bold_save);
+      g_object_unref(zp->bold_save);
     gdk_window_get_geometry(GTK_WIDGET(term)->window,NULL,NULL,NULL,NULL,&depth);
 
     zp->bold_save = gdk_pixmap_new(GTK_WIDGET(term)->window,
@@ -1459,11 +1506,11 @@ zvt_term_set_font_name (ZvtTerm *term, char *name)
   }
 
   if (rest) {
-    g_string_sprintf (outname, "%s-medium-r%s", newname->str, rest);
+    g_string_printf (outname, "%s-medium-r%s", newname->str, rest);
     font = gdk_font_load (outname->str);
     d( printf("loading normal font %s\n", outname->str) );
     
-    g_string_sprintf (outname, "%s-bold-r%s", newname->str, rest); 
+    g_string_printf (outname, "%s-bold-r%s", newname->str, rest); 
     font_bold = gdk_font_load (outname->str);
     d( printf("loading bold font %s\n", outname->str) );
     
@@ -1498,7 +1545,7 @@ zvt_term_fix_scrollbar (ZvtTerm *term)
   GTK_ADJUSTMENT(term->adjustment)->page_size =
     term->vx->vt.height - 1;
 
-  gtk_signal_emit_by_name (GTK_OBJECT (term->adjustment), "changed");
+  g_signal_emit_by_name (term->adjustment, "changed");
 }
 
 static void
@@ -1638,7 +1685,7 @@ zvt_term_button_press (GtkWidget      *widget,
 
     /* 'block' input while we're selecting text */
     if (term->input_id!=-1) {
-      gdk_input_remove(term->input_id);
+      g_source_remove(term->input_id);
       term->input_id=-1;
     }
     break;
@@ -1690,14 +1737,14 @@ zvt_term_button_press (GtkWidget      *widget,
 			NULL, NULL, 0);
       /* 'block' input while we're selecting text */
       if (term->input_id!=-1) {
-	gdk_input_remove(term->input_id);
+	g_source_remove(term->input_id);
 	term->input_id=-1;
       }
     }
     break;
   }
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -1768,7 +1815,7 @@ zvt_term_button_release (GtkWidget      *widget,
     /* re-enable input */
     if (term->input_id==-1 && term->vx->vt.childfd !=-1) {
       term->input_id =
-	gdk_input_add(term->vx->vt.childfd, GDK_INPUT_READ, zvt_term_readdata, term);
+	zvt_input_add(term->vx->vt.childfd, READ_CONDITION, zvt_term_readdata, term);
     }
 
     if (vx->selectiontype & VT_SELTYPE_MOVED) {
@@ -2099,7 +2146,7 @@ zvt_term_writechild(ZvtTerm *term, char *data, int len)
       zp->paste_len = len-length;
       memcpy(zp->paste, data+length, len-length);
       zp->paste_id =
-	gdk_input_add(term->vx->vt.keyfd, GDK_INPUT_WRITE, zvt_term_writemore, term);
+	zvt_input_add(term->vx->vt.keyfd, WRITE_CONDITION, zvt_term_writemore, term);
     } else {
       /* could also drop off offset, but its rarely likely to happen? */
       zp->paste = g_realloc(zp->paste, zp->paste_len + len - length);
@@ -2110,8 +2157,8 @@ zvt_term_writechild(ZvtTerm *term, char *data, int len)
   return length;
 }
 
-static void
-zvt_term_writemore (gpointer data, gint fd, GdkInputCondition condition)
+static gboolean
+zvt_term_writemore (GIOChannel *source, GIOCondition condition, gpointer data)
 {
   ZvtTerm *term = (ZvtTerm *)data;
   struct _zvtprivate *zp;
@@ -2128,12 +2175,13 @@ zvt_term_writemore (gpointer data, gint fd, GdkInputCondition condition)
     }
     g_free(zp->paste);
     zp->paste = 0;
-    gdk_input_remove(zp->paste_id);
+    g_source_remove(zp->paste_id);
     zp->paste_id = -1;
   } else {
     zp->paste_offset += length;
     zp->paste_len -= length;
   }
+  return TRUE;
 }
 
 
@@ -2251,9 +2299,9 @@ zvt_term_forkpty (ZvtTerm *term, int do_uwtmp_log)
   pid = vt_forkpty (&term->vx->vt, do_uwtmp_log);
   if (pid > 0) {
     term->input_id = 
-      gdk_input_add(term->vx->vt.childfd, GDK_INPUT_READ, zvt_term_readdata, term);
+      zvt_input_add(term->vx->vt.childfd, READ_CONDITION, zvt_term_readdata, term);
     term->msg_id =
-      gdk_input_add(term->vx->vt.msgfd, GDK_INPUT_READ, zvt_term_readmsg, term);
+      zvt_input_add(term->vx->vt.msgfd, READ_CONDITION, zvt_term_readmsg, term);
   }
 
   return pid;
@@ -2298,12 +2346,12 @@ zvt_term_closepty (ZvtTerm *term)
   g_return_val_if_fail (ZVT_IS_TERM (term), -1);
 
   if (term->input_id != -1) {
-    gdk_input_remove (term->input_id);
+    g_source_remove (term->input_id);
     term->input_id = -1;
   }
 
   if (term->msg_id != -1) {
-    gdk_input_remove (term->msg_id);
+    g_source_remove (term->msg_id);
     term->msg_id = -1;
   }
 
@@ -2728,7 +2776,7 @@ zvt_term_title_changed_raise (void *user_data, char *str, VTTITLE_TYPE type)
   g_return_if_fail (term != NULL);
   g_return_if_fail (ZVT_IS_TERM (term));
 
-  gtk_signal_emit(GTK_OBJECT(term), term_signals[TITLE_CHANGED], type, str);
+  g_signal_emit(term, term_signals[TITLE_CHANGED], 0, type, str);
 }
 
 /* emit the got_output signal */
@@ -2738,7 +2786,7 @@ zvt_term_got_output (ZvtTerm *term, const gchar *buffer, gint count)
     g_return_if_fail (term != NULL);
     g_return_if_fail (ZVT_IS_TERM (term));
     
-    gtk_signal_emit(GTK_OBJECT(term), term_signals[GOT_OUTPUT],
+    g_signal_emit(term, term_signals[GOT_OUTPUT], 0,
 		    buffer, count);
 }
 
@@ -2763,21 +2811,23 @@ vtx_unrender_selection (struct _vtx *vx)
  *
  * NOTE: this may impact on slow machines, but it may not also ...!
  */
-static void
-zvt_term_readdata (gpointer data, gint fd, GdkInputCondition condition)
+static gboolean
+zvt_term_readdata (GIOChannel *source, GIOCondition condition, gpointer data)
 {
   gboolean update;
   gchar buffer[4096];
   gint count, saveerrno;
   struct _vtx *vx;
   ZvtTerm *term;
+  gint fd;
 
   d( printf("zvt_term_readdata\n") );
 
+  fd = g_io_channel_unix_get_fd (source);
   term = (ZvtTerm *) data;
 
   if (term->input_id == -1)
-    return;
+    return FALSE;
 
   update = FALSE;
   vx = term->vx;
@@ -2791,8 +2841,8 @@ zvt_term_readdata (gpointer data, gint fd, GdkInputCondition condition)
     saveerrno = errno;
     vt_parse_vt (&vx->vt, buffer, count);
 
-    if (gtk_signal_handler_pending(GTK_OBJECT(term),
-				   term_signals[GOT_OUTPUT], TRUE))
+    if (g_signal_has_handler_pending(G_OBJECT(term),
+				   term_signals[GOT_OUTPUT], 0, TRUE))
 	zvt_term_got_output(term, buffer, count);
   }
 
@@ -2826,6 +2876,7 @@ zvt_term_readdata (gpointer data, gint fd, GdkInputCondition condition)
   zvt_term_fix_scrollbar (term);
 
   zvt_term_updated(term, 2);
+  return TRUE;
 }
 
 /**
@@ -2875,8 +2926,8 @@ zvt_term_feed (ZvtTerm *term, char *text, int len)
   zvt_term_updated(term, 1);
 }
 
-static void
-zvt_term_readmsg (gpointer data, gint fd, GdkInputCondition condition)
+static gboolean
+zvt_term_readmsg (GIOChannel *source, GIOCondition condition, gpointer data)
 {
   ZvtTerm *term = (ZvtTerm *)data;
 
@@ -2884,14 +2935,15 @@ zvt_term_readmsg (gpointer data, gint fd, GdkInputCondition condition)
    * it doesn't seem worth the trouble <shrug>
    */
   if (term->input_id != -1) {
-    gdk_input_remove (term->input_id);
+    g_source_remove (term->input_id);
     term->input_id=-1;
   }
 
   zvt_term_closepty (term);
 
   /* signal application FIXME: include error/non error code */
-  gtk_signal_emit (GTK_OBJECT(term), term_signals[CHILD_DIED]);
+  g_signal_emit (term, term_signals[CHILD_DIED], 0);
+  return TRUE;
 }
 
 /**
@@ -2929,7 +2981,7 @@ zvt_term_set_background (ZvtTerm *terminal, char *pixmap_file,
     zvt_term_background_set_pixmap_file(b, pixmap_file);
   } else if (transparent) {
     b = zvt_term_background_new(terminal);  
-    zvt_term_background_set_pixmap_atom(b, GDK_ROOT_PARENT(),
+    zvt_term_background_set_pixmap_atom(b, gdk_get_default_root_window(),
 					gdk_atom_intern("_XROOTPMAP_ID", TRUE));
     zvt_term_background_set_translate(b, ZVT_BGTRANSLATE_ROOT, 0, 0);
   }
@@ -3051,7 +3103,7 @@ vt_draw_text(void *user_data, struct vt_line *line, int row, int col, int len, i
       f = term->font;
       overstrike = 1;
       if (zp && zp->bold_save) {
-	gdk_draw_pixmap(zp->bold_save, term->fore_gc, GTK_WIDGET(term)->window,
+	gdk_draw_drawable(zp->bold_save, term->fore_gc, GTK_WIDGET(term)->window,
 			x + offx + len*term->charwidth, offy + row*term->charheight,
 			0, 0, 1, term->charheight);
       }
@@ -3261,7 +3313,7 @@ vt_draw_text(void *user_data, struct vt_line *line, int row, int col, int len, i
   }
   
   if (overstrike && zp && zp->bold_save) {
-    gdk_draw_pixmap(GTK_WIDGET(term)->window,
+    gdk_draw_drawable(GTK_WIDGET(term)->window,
 		    term->fore_gc,
 		    zp->bold_save,
 		    0, 0,
@@ -3304,7 +3356,7 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
   offy = widget->style->ythickness;
 
   /* "scroll" area */
-  gdk_draw_pixmap(widget->window,
+  gdk_draw_drawable(widget->window,
 		  term->scroll_gc, /* must use this to generate expose events */
 		  widget->window,
 		  offx, offy+(firstrow+offset)*term->charheight,
@@ -3372,7 +3424,7 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
 static void
 vt_selection_changed (void *user_data)
 {
-  gtk_signal_emit_by_name (GTK_OBJECT (user_data), "selection_changed");
+  g_signal_emit_by_name (user_data, "selection_changed");
 }
 
 
@@ -3613,3 +3665,34 @@ zvt_term_get_buffer(ZvtTerm *term, int *len, int type, int sx, int sy, int ex, i
 
   return data;
 }
+
+
+static AtkObject *
+zvt_term_get_accessible (GtkWidget *widget)
+{
+  static gboolean first_time = TRUE;
+  AtkRegistry *registry;
+  AtkObjectFactory *factory;  
+  GType derived_type;
+  GType derived_atk_type;
+
+  if (first_time)
+    {
+      /* Determine whether accessibility is enabled by checking accessible
+	 created for parent */
+
+      registry = atk_get_default_registry ();
+      derived_type = g_type_parent (ZVT_TYPE_TERM);
+      factory = atk_registry_get_factory (registry,
+					 derived_type);
+      derived_atk_type = atk_object_factory_get_accessible_type (factory);
+      if (g_type_is_a(derived_atk_type, GTK_TYPE_ACCESSIBLE))
+	{
+	  atk_registry_set_factory_type (registry,
+					 ZVT_TYPE_TERM, ZVT_TYPE_ACCESSIBLE_FACTORY);
+	}
+      first_time = FALSE;
+    }
+  return GTK_WIDGET_CLASS(parent_class)->get_accessible (widget);
+}
+
